"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

require("core-js/modules/web.dom.iterable");

var _path = _interopRequireDefault(require("path"));

var _Disk = _interopRequireDefault(require("@polkadot/db/Disk"));

var _Memory = _interopRequireDefault(require("@polkadot/db/Memory"));

var _trieDb = _interopRequireDefault(require("@polkadot/trie-db"));

var _logger = _interopRequireDefault(require("@polkadot/util/logger"));

var _toHex = _interopRequireDefault(require("@polkadot/util/u8a/toHex"));

var _block = _interopRequireDefault(require("./block"));

var _state = _interopRequireDefault(require("./state"));

// Copyright 2017-2018 @polkadot/client-chains authors & contributors
// This software may be modified and distributed under the terms
// of the ISC license. See the LICENSE file for details.
const SPINNER = ['|', '/', '-', '\\'];
const PREPEND = '                                     ';
const l = (0, _logger.default)('db');

class Dbs {
  constructor({
    db
  }, chain) {
    this.blocks = void 0;
    this.state = void 0;
    const isDisk = db.type === 'disk';
    const dbPath = isDisk ? _path.default.join(db.path, 'chains', chain.id, (0, _toHex.default)(chain.genesisRoot)) : '';
    this.blocks = (0, _block.default)(isDisk // NOTE blocks compress very well
    ? new _Disk.default(dbPath, 'block.db', {
      isCompressed: true
    }) : new _Memory.default());
    this.state = (0, _state.default)(new _trieDb.default(isDisk // NOTE state RLP does not compress well here
    ? new _Disk.default(dbPath, 'state.db', {
      isCompressed: false
    }) : new _Memory.default()));

    if (db.compact) {
      this.maintain('block', this.blocks.db);
      this.maintain('state', this.state.db);
    }

    this.blocks.db.open();
    this.state.db.open();
  }

  maintain(name, db) {
    l.log(`compacting ${name} database`);
    let spin = 0;
    let lastUpdate = 0;
    db.maintain(progress => {
      const now = Date.now();

      if (now - lastUpdate > 200) {
        const percent = `      ${progress.percent.toFixed(2)}`.slice(-6);
        const keys = (progress.keys / 1000).toFixed(2);
        process.stdout.write(`${PREPEND}${SPINNER[spin % SPINNER.length]} ${percent}%, ${keys}k keys\r`);
        lastUpdate = now;
        spin++;
      }
    });
  }

}

exports.default = Dbs;